/******************************************************************************

	COPYRIGHT(C) JONAS 'SORTIE' TERMANSEN 2011.

	This file is part of LibMaxsi.

	LibMaxsi is free software: you can redistribute it and/or modify it under
	the terms of the GNU Lesser General Public License as published by the Free
	Software Foundation, either version 3 of the License, or (at your option)
	any later version.

	LibMaxsi is distributed in the hope that it will be useful, but WITHOUT ANY
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
	FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for
	more details.

	You should have received a copy of the GNU Lesser General Public License
	along with LibMaxsi. If not, see <http://www.gnu.org/licenses/>.

	fcntl.h
	File control options.

******************************************************************************/

/* TODO: POSIX-1.2008 compliance is only partial */

#ifndef	_FCNTL_H
#define	_FCNTL_H 1

#include <features.h>

__BEGIN_DECLS

/* TODO: F_* missing here */

/* TODO: FD_CLOEXEC missing here */

/* TODO: F_RDLCK, F_UNLCK, F_WRLCK missing here */

#ifndef SEEK_SET
#define SEEK_SET 0 /* Seek from beginning of file.  */
#endif

#ifndef SEEK_CUR
#define SEEK_CUR 1 /* Seek from current position.  */
#endif

#ifndef SEEK_SET
#define SEEK_SET 2 /* Seek from end of file.  */
#endif


/* TODO: Keep these aligned with those in the Sortix kernel */
#define O_ACCMODE	   0003
#define O_RDONLY	     00
#define O_WRONLY	     01
#define O_RDWR		     02
#define O_CREAT		   0100
#define O_EXCL		   0200
#define O_NOCTTY	   0400
#define O_TRUNC		  01000
#define O_APPEND	  02000
#define O_NONBLOCK	  04000
#define O_NDELAY	O_NONBLOCK
#define O_SYNC	       04010000
#define O_FSYNC		 O_SYNC
#define O_ASYNC		 020000

#ifndef _MODE_T_DECL
#define _MODE_T_DECL
typedef __mode_t mode_t;
#endif

#ifndef _MODE_T_VALUES_DECL
#define _MODE_T_VALUES_DECL
#define S_IRUSR	__S_IREAD       /* Read by owner.  */
#define S_IWUSR	__S_IWRITE      /* Write by owner.  */
#define S_IXUSR	__S_IEXEC       /* Execute by owner.  */
/* Read, write, and execute by owner.  */
#define S_IRWXU	(__S_IREAD|__S_IWRITE|__S_IEXEC)

#define S_IRGRP	(S_IRUSR >> 3)  /* Read by group.  */
#define S_IWGRP	(S_IWUSR >> 3)  /* Write by group.  */
#define S_IXGRP	(S_IXUSR >> 3)  /* Execute by group.  */
/* Read, write, and execute by group.  */
#define S_IRWXG	(S_IRWXU >> 3)

#define S_IROTH	(S_IRGRP >> 3)  /* Read by others.  */
#define S_IWOTH	(S_IWGRP >> 3)  /* Write by others.  */
#define S_IXOTH	(S_IXGRP >> 3)  /* Execute by others.  */
/* Read, write, and execute by others.  */
#define S_IRWXO	(S_IRWXG >> 3)

#define	__S_ISUID	04000	/* Set user ID on execution.  */
#define	__S_ISGID	02000	/* Set group ID on execution.  */
#define	__S_ISVTX	01000	/* Save swapped text after use (sticky).  */
#define	__S_IREAD	0400	/* Read by owner.  */
#define	__S_IWRITE	0200	/* Write by owner.  */
#define	__S_IEXEC	0100	/* Execute by owner.  */
#endif


/* TODO: AT_FDCWD missing here */
/* TODO: AT_EACCESS missing here */
/* TODO: AT_SYMLINK_NOFOLLOW missing here */
/* TODO: AT_SYMLINK_FOLLOW missing here */
/* TODO: AT_REMOVEDIR missing here */

/* TODO: POSIX_FADV_* missing here */

#ifndef _PID_T_DECL
#define _PID_T_DECL
typedef __pid_t pid_t;
#endif


struct _flock
{
	short l_type; /* Type of lock; F_RDLCK, F_WRLCK, F_UNLCK. */
	short l_whence; /* Type of lock; F_RDLCK, F_WRLCK, F_UNLCK. */
	off_t l_start; /* Relative offset in bytes. */
	off_t l_len; /* Size; if 0 then until EOF. */
	pid_t l_pid; /* Process ID of the process holding the lock; returned with F_GETLK. */
};

typedef struct _flock flock;

/* TODO: These are not implemented in libmaxsi/sortix yet. */
#ifndef SORTIX_UNIMPLEMENTED
int creat(const char* path, mode_t mode);
int fcntl(int fd, int cmd, ...);
int open(const char* path, int oflag, ...);
int openat(int fd, const char* path, int oflag, ...);
#endif

__END_DECLS

#endif

