/******************************************************************************

	COPYRIGHT(C) JONAS 'SORTIE' TERMANSEN 2011.

	This file is part of Sortix.

	Sortix is free software: you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation, either version 3 of the License, or (at your option) any later
	version.

	Sortix is distributed in the hope that it will be useful, but WITHOUT ANY
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
	FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
	details.

	You should have received a copy of the GNU General Public License along
	with Sortix. If not, see <http://www.gnu.org/licenses/>.

	panic.cpp
	Displays an error whenever something critical happens.

******************************************************************************/

#include "platform.h"
#include <libmaxsi/string.h>
#include <libmaxsi/memory.h>
#include "globals.h"
#include "iprintable.h"
#include "log.h"
#include "panic.h"

#include "uart.h"

using namespace Maxsi;

namespace Sortix
{
	DEFINE_GLOBAL_OBJECT(PanicClass, Panic);

	void PanicClass::Init()
	{
		Colors[Line] = (COLOR8_RED << 0) + (COLOR8_BLACK << 4);
		Print("                                                       _                        ");
		Print("                                                      / \\                       ");
		Print("                  /\\    /\\                           /   \\                      ");
		Print("                 /  \\  /  \\                          |   |                      ");
		Print("                /    \\/    \\                         |   |                      ");
		Print("               |  X    X    \\_______________________ /   |                      ");
		Print("               |                                         |                      ");
		Print("               | _________                               /                      ");
		Print("                \\                                       /                       ");
		Print("                  ------       ---------------      ---/                        ");
		Print("                       /       \\             /      \\                           ");
		Print("                      /         \\           /        \\                          ");
		Print("                     /           \\         /          \\                         ");
		Print("                    /_____________\\       /____________\\                        ");
		Print("                                                                                ");
		Print("                                                                                ");
		Colors[Line] = (COLOR8_RED << 0) + (COLOR8_DARK_GREY << 4);
#ifdef JSSORTIX
		Print("                         (NOT VERY) RED MAXSI OF DEATH                          ");
#else
		Print("                              RED MAXSI OF DEATH                                ");
#endif
		Colors[Line] = (COLOR8_RED << 0) + (COLOR8_BLACK << 4);
		Print("                                                                                ");
	}

	void PanicClass::Halt()
	{
		UpdateScreen();
		#ifdef JSSORTIX
		JSSortix::Exit();
		#endif
		HaltKernel();
	}

	extern "C" void Panic(const char* Error)
	{
		GPanic->Init();
		GPanic->Print(Error);
		GPanic->Halt();	
	}

	extern "C" void PanicF(const char* Format, ...)
	{
		GPanic->Init();
		va_list list;
		va_start(list, Format);
		GPanic->PrintFV(Format, list);
		va_end(list);
		GPanic->Halt();	
	}
}
